"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CollapsibleTabContext = void 0;
exports.CollapsibleTabProvider = CollapsibleTabProvider;
var _react = _interopRequireWildcard(require("react"));
var _reactNativeReanimated = require("react-native-reanimated");
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
const CollapsibleTabContext = /*#__PURE__*/(0, _react.createContext)({});
exports.CollapsibleTabContext = CollapsibleTabContext;
const flatListNodes = {};
const scrollViewNodes = {};
const scrollPositions = {};
const DEFAULT_LAYOUT = {
  x: 0,
  y: 0,
  width: 0,
  height: 0
};
const DEFAULT_CONTENT_OFFSET = {
  x: 0,
  y: 0
};
const DEFAULT_LAYOUTS = {
  container: DEFAULT_LAYOUT,
  header: DEFAULT_LAYOUT,
  tabBar: DEFAULT_LAYOUT
};
function CollapsibleTabProvider(_ref) {
  let {
    children
  } = _ref;
  const containerLayout = (0, _reactNativeReanimated.useSharedValue)({
    ...DEFAULT_LAYOUT
  });
  const headerLayout = (0, _reactNativeReanimated.useSharedValue)({
    ...DEFAULT_LAYOUT
  });
  const tabBarLayout = (0, _reactNativeReanimated.useSharedValue)({
    ...DEFAULT_LAYOUT
  });
  const contentOffset = (0, _reactNativeReanimated.useSharedValue)({
    ...DEFAULT_CONTENT_OFFSET
  });
  const [layouts, setLayouts] = (0, _react.useState)({
    ...DEFAULT_LAYOUTS
  });
  const stateScrollPositions = (id, offset) => {
    scrollPositions[id] = offset;
  };
  const stateLayout = (name, value) => {
    setLayouts(prev => ({
      ...prev,
      [name]: value
    }));
  };
  const onLayoutHeader = async event => {
    const layout = event.nativeEvent.layout;
    headerLayout.value = layout;
    stateLayout('header', layout);
  };
  const onLayoutTabBar = async event => {
    const layout = event.nativeEvent.layout;
    tabBarLayout.value = layout;
    stateLayout('tabBar', layout);
  };
  const onLayoutContainer = async event => {
    const layout = event.nativeEvent.layout;
    containerLayout.value = layout;
    stateLayout('container', layout);
  };

  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  const fixScrolls = withOutId => {
    const getOffset = ownPosition => {
      const offset = contentOffset.value.y >= headerLayout.value.height ? Math.max(ownPosition, headerLayout.value.height) : contentOffset.value.y;
      return offset;
    };
    Object.keys(flatListNodes).filter(id => id !== withOutId).forEach(id => {
      var _scrollPositions$Numb;
      const node = flatListNodes[Number(id)];
      const ownPosition = ((_scrollPositions$Numb = scrollPositions[Number(id)]) === null || _scrollPositions$Numb === void 0 ? void 0 : _scrollPositions$Numb.y) ?? 0;
      const offset = getOffset(ownPosition) + 0.001;
      node === null || node === void 0 ? void 0 : node.scrollToOffset({
        offset: offset,
        animated: false
      });
    });
    Object.keys(scrollViewNodes).filter(id => id !== withOutId).forEach(id => {
      var _scrollPositions$Numb2;
      const node = scrollViewNodes[Number(id)];
      const ownPosition = ((_scrollPositions$Numb2 = scrollPositions[Number(id)]) === null || _scrollPositions$Numb2 === void 0 ? void 0 : _scrollPositions$Numb2.y) ?? 0;
      const offset = getOffset(ownPosition) + 0.001;
      node === null || node === void 0 ? void 0 : node.scrollTo({
        y: offset,
        animated: false
      });
    });
  };
  const lockScrolls = () => {
    Object.values({
      ...flatListNodes,
      ...scrollViewNodes
    }).forEach(node => {
      node === null || node === void 0 ? void 0 : node.setNativeProps({
        scrollEnabled: false
      });
    });
  };
  const enableScrolls = () => {
    Object.values({
      ...flatListNodes,
      ...scrollViewNodes
    }).forEach(node => {
      node === null || node === void 0 ? void 0 : node.setNativeProps({
        scrollEnabled: true
      });
    });
  };
  const enableScroll = id => {
    const node = flatListNodes[id] || scrollViewNodes[id];
    node === null || node === void 0 ? void 0 : node.setNativeProps({
      scrollEnabled: true
    });
  };
  const scrollHandler = id => (0, _reactNativeReanimated.useAnimatedScrollHandler)({
    onScroll: event => {
      contentOffset.value = event.contentOffset;
      (0, _reactNativeReanimated.runOnJS)(stateScrollPositions)(id, event.contentOffset);
    },
    onEndDrag: () => (0, _reactNativeReanimated.runOnJS)(fixScrolls)(id),
    onMomentumEnd: () => (0, _reactNativeReanimated.runOnJS)(fixScrolls)(id)
  });
  const rContainerStyle = (0, _reactNativeReanimated.useAnimatedStyle)(() => ({
    opacity: headerLayout.value.height ? (0, _reactNativeReanimated.withDelay)(100, (0, _reactNativeReanimated.withTiming)(1, {
      duration: 0
    })) : 0
  }));
  const rHeaderStyle = (0, _reactNativeReanimated.useAnimatedStyle)(() => ({
    transform: [{
      translateY: -Math.min(headerLayout.value.height, contentOffset.value.y)
    }]
  }));
  const rTabBarStyle = (0, _reactNativeReanimated.useAnimatedStyle)(() => {
    var _headerLayout$value;
    return {
      top: (_headerLayout$value = headerLayout.value) === null || _headerLayout$value === void 0 ? void 0 : _headerLayout$value.height,
      transform: [{
        translateY: -Math.min(headerLayout.value.height, contentOffset.value.y)
      }]
    };
  });
  return /*#__PURE__*/_react.default.createElement(CollapsibleTabContext.Provider, {
    value: {
      rContainerStyle,
      rHeaderStyle,
      rTabBarStyle,
      layouts,
      flatListNodes,
      scrollViewNodes,
      contentOffset,
      containerLayout,
      headerLayout,
      tabBarLayout,
      onLayoutContainer,
      onLayoutHeader,
      onLayoutTabBar,
      fixScrolls,
      lockScrolls,
      enableScrolls,
      enableScroll,
      scrollHandler
    }
  }, children);
}
//# sourceMappingURL=CollapsibleTabContext.js.map