import React, { createContext, useState } from 'react';
import { runOnJS, useAnimatedScrollHandler, useAnimatedStyle, useSharedValue, withDelay, withTiming } from 'react-native-reanimated';
export const CollapsibleTabContext = /*#__PURE__*/createContext({});
const flatListNodes = {};
const scrollViewNodes = {};
const scrollPositions = {};
const DEFAULT_LAYOUT = {
  x: 0,
  y: 0,
  width: 0,
  height: 0
};
const DEFAULT_CONTENT_OFFSET = {
  x: 0,
  y: 0
};
const DEFAULT_LAYOUTS = {
  container: DEFAULT_LAYOUT,
  header: DEFAULT_LAYOUT,
  tabBar: DEFAULT_LAYOUT
};
export function CollapsibleTabProvider(_ref) {
  let {
    children
  } = _ref;
  const containerLayout = useSharedValue({
    ...DEFAULT_LAYOUT
  });
  const headerLayout = useSharedValue({
    ...DEFAULT_LAYOUT
  });
  const tabBarLayout = useSharedValue({
    ...DEFAULT_LAYOUT
  });
  const contentOffset = useSharedValue({
    ...DEFAULT_CONTENT_OFFSET
  });
  const [layouts, setLayouts] = useState({
    ...DEFAULT_LAYOUTS
  });
  const stateScrollPositions = (id, offset) => {
    scrollPositions[id] = offset;
  };
  const stateLayout = (name, value) => {
    setLayouts(prev => ({
      ...prev,
      [name]: value
    }));
  };
  const onLayoutHeader = async event => {
    const layout = event.nativeEvent.layout;
    headerLayout.value = layout;
    stateLayout('header', layout);
  };
  const onLayoutTabBar = async event => {
    const layout = event.nativeEvent.layout;
    tabBarLayout.value = layout;
    stateLayout('tabBar', layout);
  };
  const onLayoutContainer = async event => {
    const layout = event.nativeEvent.layout;
    containerLayout.value = layout;
    stateLayout('container', layout);
  };

  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  const fixScrolls = withOutId => {
    const getOffset = ownPosition => {
      const offset = contentOffset.value.y >= headerLayout.value.height ? Math.max(ownPosition, headerLayout.value.height) : contentOffset.value.y;
      return offset;
    };
    Object.keys(flatListNodes).filter(id => id !== withOutId).forEach(id => {
      var _scrollPositions$Numb;
      const node = flatListNodes[Number(id)];
      const ownPosition = ((_scrollPositions$Numb = scrollPositions[Number(id)]) === null || _scrollPositions$Numb === void 0 ? void 0 : _scrollPositions$Numb.y) ?? 0;
      const offset = getOffset(ownPosition) + 0.001;
      node === null || node === void 0 ? void 0 : node.scrollToOffset({
        offset: offset,
        animated: false
      });
    });
    Object.keys(scrollViewNodes).filter(id => id !== withOutId).forEach(id => {
      var _scrollPositions$Numb2;
      const node = scrollViewNodes[Number(id)];
      const ownPosition = ((_scrollPositions$Numb2 = scrollPositions[Number(id)]) === null || _scrollPositions$Numb2 === void 0 ? void 0 : _scrollPositions$Numb2.y) ?? 0;
      const offset = getOffset(ownPosition) + 0.001;
      node === null || node === void 0 ? void 0 : node.scrollTo({
        y: offset,
        animated: false
      });
    });
  };
  const lockScrolls = () => {
    Object.values({
      ...flatListNodes,
      ...scrollViewNodes
    }).forEach(node => {
      node === null || node === void 0 ? void 0 : node.setNativeProps({
        scrollEnabled: false
      });
    });
  };
  const enableScrolls = () => {
    Object.values({
      ...flatListNodes,
      ...scrollViewNodes
    }).forEach(node => {
      node === null || node === void 0 ? void 0 : node.setNativeProps({
        scrollEnabled: true
      });
    });
  };
  const enableScroll = id => {
    const node = flatListNodes[id] || scrollViewNodes[id];
    node === null || node === void 0 ? void 0 : node.setNativeProps({
      scrollEnabled: true
    });
  };
  const scrollHandler = id => useAnimatedScrollHandler({
    onScroll: event => {
      contentOffset.value = event.contentOffset;
      runOnJS(stateScrollPositions)(id, event.contentOffset);
    },
    onEndDrag: () => runOnJS(fixScrolls)(id),
    onMomentumEnd: () => runOnJS(fixScrolls)(id)
  });
  const rContainerStyle = useAnimatedStyle(() => ({
    opacity: headerLayout.value.height ? withDelay(100, withTiming(1, {
      duration: 0
    })) : 0
  }));
  const rHeaderStyle = useAnimatedStyle(() => ({
    transform: [{
      translateY: -Math.min(headerLayout.value.height, contentOffset.value.y)
    }]
  }));
  const rTabBarStyle = useAnimatedStyle(() => {
    var _headerLayout$value;
    return {
      top: (_headerLayout$value = headerLayout.value) === null || _headerLayout$value === void 0 ? void 0 : _headerLayout$value.height,
      transform: [{
        translateY: -Math.min(headerLayout.value.height, contentOffset.value.y)
      }]
    };
  });
  return /*#__PURE__*/React.createElement(CollapsibleTabContext.Provider, {
    value: {
      rContainerStyle,
      rHeaderStyle,
      rTabBarStyle,
      layouts,
      flatListNodes,
      scrollViewNodes,
      contentOffset,
      containerLayout,
      headerLayout,
      tabBarLayout,
      onLayoutContainer,
      onLayoutHeader,
      onLayoutTabBar,
      fixScrolls,
      lockScrolls,
      enableScrolls,
      enableScroll,
      scrollHandler
    }
  }, children);
}
//# sourceMappingURL=CollapsibleTabContext.js.map